#pragma once

#include <avr/pgmspace.h>


typedef signed int INT16S;
typedef signed int INT16;
typedef unsigned int UINT16;
typedef unsigned long int UINT32;
typedef signed long  int INT32S;
typedef unsigned int INT16US;
typedef unsigned char INT8US;
typedef unsigned int WORD;
typedef unsigned char UINT8;
typedef unsigned char BOOL;   
typedef unsigned char BYTE;

#define TRUE		1
#define FALSE		0

#define ON			1
#define OFF			0





#define SetBit(PORT,ibit)        (PORT=PORT| ((1<<ibit))) 
#define ClearBit(PORT, ibit)    (PORT=PORT&(~(1<<ibit))) 



#include <avr/io.h>
#include <avr/interrupt.h>


#define MAIN_CLK	11059200	// 11.0592MHz
#define _USE_SCI0_
#define _USE_SCI1_
#define _USE_KEY4x4_
#define _USE_CLCD_
#define _USE_GLCD_
#define _USE_ADC_
#define _USE_FND4_
#define _USE_ROTARY_




//////////////////////////////////////////////////////////////////////////
//
//	UART
//
//////////////////////////////////////////////////////////////////////////


#define RX_BUFFER_SIZE 100

#define	COMM_STX	0x02
#define COMM_ETX	0x03

#define TXB8	0
#define RXB8	1
#define UPE		2
#define OVR		3
#define FE		4
#define UDRE	5 
#define RXC		7

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<OVR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)



//! USART RX Information
struct _USART_RX{
	char rx_buffer[RX_BUFFER_SIZE];			/*!< RX Buffer */
	unsigned char rx_wr_index1,rx_rd_index1;	
};

//! USART TX Information
struct _USART_TX{
	unsigned char wr_index1,rd_index1;	
	BOOL bComplete;
};



char g_temp[64];

#ifdef _USE_SCI0_

struct _USART_TX   structTX0Info;
struct _USART_RX	structRX0Info;
void OpenSCI0(UINT16 iBaud);				/*!< sci 0 ʱȭ */
INT16S GetByte0(void);
BOOL PutChar0(char c);						/*!< sci 0 1 Ʈ  */
void PutBytes0(BYTE *data, int iLength);	/*!< sci 0  Ʈ  */
void PutData0(BYTE *data, int iLength);		/*!< sci 0 ڵ STX, ETX, CRC Ͽ  */

//!  ڿ 
BOOL PutString0(char  *string );


#endif

#ifdef _USE_SCI1_
struct _USART_TX   structTX1Info;
struct _USART_RX	structRX1Info;

void InitSCI1(UINT16 iBaud);
INT16S GetByte1(void);
BOOL PutByte1(unsigned char c);
BOOL PutString1(char  *string );

#endif




#pragma region KEY 4x4
//////////////////////////////////////////////////////////////////////////
//
//	KEY 4x4
//
//////////////////////////////////////////////////////////////////////////

/*
PC-> AVR û:
		02 10 01 xx xx xx CRC 03

AVR-> PC :
		02 11 KEY_H KEY_L xx xx CRC 03
		KEY_H : Ű  16Ʈ  8 Ʈ ( SW16 ~ SW9  ش)
		KEY_L : Ű  16Ʈ  8 Ʈ ( SW8  ~ SW1  ش)


*/

//enum 
//{	
//	SW1_1=0,
//	SW1_2,
//	SW1_3,
//	SW1_4,
//
//	SW2_1,
//	SW2_2,
//	SW2_3,
//	SW2_4,
//
//	SW3_1,
//	SW3_2,
//	SW3_3,
//	SW3_4,
//
//	SW4_1,
//	SW4_2,
//	SW4_3,
//	SW4_4,
//
//	SW_RESERVED	
//} KEY_4x4_NAME;



#define KEY4x4_PORT	PORTA
#define KEY4x4_PORT_IN	PINA
#define KEY4x4_PORT_DIR	DDRA


#ifdef _USE_KEY4x4_
void InitKey4x4();
UINT16 GetKey4x4();
void SendKey4x4(UINT16 uidata);

#endif
#pragma endregion KEY 4x4


#ifdef _USE_CLCD_


#define CLCD_RS 1 
#define CLCD_RW 3 
#define CLCD_EA 5
#define CLCD_DATA_PORT	PORTB
#define CLCD_DATA_PORT_DIR	DDRB

#define CLCD_CONTROL_PORT	PORTA
#define CLCD_CONTROL_PORT_DIR	DDRA



void CLCD_XY(BYTE x1, BYTE y1); 
void CLCD_Command(BYTE data); 
void CLCD_On(); 
BYTE CLCD_BusyCheck(); 
void CLCD_PutString(BYTE x, BYTE y, char  *str); 
void CLCD_PutChar(BYTE chr); 
void CLCD_Clear(); 
void CLCD_Init();


#endif





#ifdef _USE_GLCD_


#define GLCD_CS1 0	//  LCD 
#define GLCD_CS2 1	//  LCD 
#define GLCD_RST 2	// RESET
#define GLCD_RW 3	// Read/Write 
#define GLCD_RS 4	// Command/Data
#define GLCD_EA 5	// Enable





#define SET_EA	SetBit(GLCD_CONTROL_PORT,GLCD_EA)
#define CLR_EA	ClearBit(GLCD_CONTROL_PORT,GLCD_EA)

#define SET_CS1	SetBit(GLCD_CONTROL_PORT,GLCD_CS1);
#define CLR_CS1	ClearBit(GLCD_CONTROL_PORT,GLCD_CS1);
#define SET_CS2	SetBit(GLCD_CONTROL_PORT,GLCD_CS2);
#define CLR_CS2	ClearBit(GLCD_CONTROL_PORT,GLCD_CS2);

#define SET_RS	SetBit(GLCD_CONTROL_PORT,GLCD_RS);
#define CLR_RS	ClearBit(GLCD_CONTROL_PORT,GLCD_RS);


#define GLCD_DATA_PORT	PORTA		// DATA 
#define GLCD_DATA_PORT_DIR	DDRA	// DATA  
#define GLCD_DATA_INPUT	PINA		// DATA Է 


#define GLCD_CONTROL_PORT	PORTB	// CONTROL 
#define GLCD_CONTROL_PORT_DIR	DDRB	// CONTROL  


struct _POSITION_ 
{
	UINT8 x;	// X ǥ
	UINT8 y;	// Y ǥ
};

// GLCD  ü
struct _GLCD_INFO_ 
{
	struct _POSITION_ pos;			//    ǥ
	struct _POSITION_ posCursor;	// Ŀ ġ
	UINT8 ShowCursor;				// Ŀ  /ƴ
};



struct _GLCD_INFO_ g_GLCD;	// GLCD ü



void GLCD_Init();			// GLCD ʱȭ
void GLCD_Clear();			// GLCD ȭ ʱȭ
void GLCD_Command(BYTE byteSignal,BYTE byteCommand);	// COMMAND
void GLCD_SetXY(BYTE x, BYTE y);		// ǥ ̵
void GLCD_SetXY_ROW(struct _POSITION_ pos,BYTE row);	// DATA  ǥ ̵

void GLCD_WriteData(BYTE byteSignal,BYTE byteChar,BYTE bBlock);	//  
BYTE GLCD_ReadData(BYTE byteSignal);							//  б
void GLCD_English(BYTE Ecode,BYTE bBlock);						//  
void GLCD_Korean(UINT16 Kcode,BYTE byteBlock)	;				// ѱ 
void GLCD_String(BYTE x,BYTE y,BYTE const *string);				// ڿ 
void GLCD_ShowCursor(UINT8 _show);							// Ŀ show/hide Ѵ.
void GLCD_RemoveCursor();									//  Ŀ Ⱥ̰ Ѵ.( Ѵ)


//  
void GLCD_SetPixel(int x1,int y1,BOOL _b);

//  ׸
void GLCD_DOT_Line(int x1,int y1,int x2,int y2);

//  ׸
void GLCD_DOT_DrawCircle(int x1,int y2,int _size);//

// 簢 ׸
void GLCD_DrawRect(int x1,int y1,int x2,int y2);


#endif






#ifdef _USE_ADC_

#define ADC_VREF_TYPE (0x40 | 0x01 ) //  : AVCC

// ADC ʱȭ
void InitADC();



#endif


#ifdef _USE_FND4_

// FND 8-segment DATA ȣ
#define FND4_DATA	PORTB
#define FND4_DATA_DIR	DDRB

// FND ڸ  ȣ
#define FND4_CONTROL_PORT	PORTA
#define FND4_CONTROL_DIR	DDRA

// FND ڸ BIT
#define FND4_SEL1_BIT	6
#define FND4_SEL2_BIT	5
#define FND4_SEL3_BIT	4
#define FND4_SEL4_BIT	7




//	FND   
BYTE g_FND[4] ;

// FND ǥ 
BYTE FND4_DISPLAY_DATA[10];
 

// FND4 ʱȭ
void InitFND4();

// FND4 ϱ
void DisplayFND4(BYTE b1,BYTE b2,BYTE b3,BYTE b4);

#endif


#ifdef _USE_ROTARY_

#define ROTARY_DATA_PORT 	PINA
#define ROTARY_DATA_DIR		DDRA




void InitRotary();
BYTE GetRotaryInt();


#endif



void DelayUS(BYTE time_us)	;

void DelayMS(unsigned int time_ms)	;


